from logger import Logger
from drivers import select_driver


class EventServer(Logger):
    """
    The event server uses an event driver to receive events, which are
    delegated to a area tree (and eventually to gesture trackers).
    """
    def __init__(self, root_area=None):
        # Root area to which events are delegated
        self.root_area = root_area

        # Driver implementation that will be serving events
        self.event_driver = select_driver(self)

    def get_root_area(self):
        return self.root_area

    def set_root_area(self, area):
        self.root_area = area

    def delegate_event(self, event):
        """
        Delegate an event that has been triggered by the event driver to the
        area tree.
        """
        if self.root_area.contains_event(event):
            event.set_root_area(self.root_area)
            self.root_area.delegate_event(event)

    def start(self):
        """
        Start the event loop. A root area is needed to be able to delegate
        events, so check if it exists first.
        """
        if not self.root_area:
            raise ValueError('Cannot start event server without root area.')

        self.event_driver.start()

    def stop(self):
        """
        Stop the event loop.
        """
        self.event_driver.stop()

    def run(self):
        """
        Execute a single loop iteration of the event driver.
        """
        self.event_driver.run()
